#ifndef _LEAR_MARK_IMAGE_H_
#define _LEAR_MARK_IMAGE_H_

#include <string>
#include <lear/classifier/ppresult.h>

namespace lear {
  /**
  Draws bounding boxes and (optionally) the classifier score for detected
  pedestrians on top of each input image.
  */
  struct MarkImage : public PostProcessResult {
    typedef PostProcessResult                    Parent;
    typedef blitz::TinyVector<unsigned char,3>   Color;
    typedef std::vector<Color>                   Colors;

    MarkImage(std::string const &outimage_,
              bool const outfileIsDir_,
              bool const hidescore_,
              Colors borderColors=Colors());

    // filename is the input file to be annotated
    virtual void write(const_iterator s, const_iterator e, 
                       RGBImage    const &inimage,
                       std::string const &infile);

    void write(std::vector<const_iterator> const &starts, 
               std::vector<const_iterator> const &ends,
               std::vector<Colors>         const &colors, 
               RGBImage                    const &inimage,
               std::string                 const &infile);

    bool static verifyImageOut(std::string const outimage_,
                               bool const outfileIsDir_);

    virtual std::string toString() { 
      return "Mark Image Post Process Result";
    }

  protected:
    std::string const outimage_;
    bool        const outfileIsDir_;
    bool        const hidescore_;
    Colors      const bordercolors_;
  };
}
#endif // _LEAR_MARK_IMAGE_H_
