#ifndef _LEAR_RESULT_HOLDER_H_
#define _LEAR_RESULT_HOLDER_H_

#include <list>
#include <lear/classifier/detectinfo.h>
#include <lear/classifier/processresult.h>
#include <lear/classifier/ppresult.h>

namespace lear {
  class ResultHolder {
  public:
    typedef std::list<ProcessResult*>           ProcessCont;
    typedef std::list<PostProcessResult*>       PostProcessCont;
    typedef std::list<PostProcessCont>          ContCont;

    ResultHolder(){}
    ~ResultHolder();

    void push_back(ProcessResult* toadd) {
      processcont_.push_back(toadd);
      postcont_.push_back(PostProcessCont());
    }

    void push2last(PostProcessResult* pp) {
      ContCont::iterator i = postcont_.end();
      --i;
      i->push_back(pp);
    }

    void push_back(ProcessResult* toadd, PostProcessResult* pp) {
      push_back(toadd) ;
      push2last(pp);
    }

    void print(std::ostream& o) const ;

    /** Prepare contained ProcessResult's for processing a new image.
      * User must invoke this function before processing a new image.
      * Also, user must invoke write or suppress_nonmax to actually
      * write detection results.
      */
    void clear();

    /** Called for each tested detector window.  Each of the ProcessResult
      * objects is responsible for caching whatever detections they wish
      * to retain.
      */
    bool operator()(DetectInfo const &r);

    /** For each ProcessResult pr, pr.doit() is used to perform
      * any final filtering of results.  The resulting detections are then
      * supplied to all of the PostProcessResult objects associated with pr.
      */
    void write(PostProcessResult::RGBImage const &inimage,
               std::string                 const &infile="");

  protected:
    ProcessCont     processcont_;
    ContCont        postcont_;
  };
}
#endif // _LEAR_RESULT_HOLDER_H_
