#ifndef _LEAR_CLASSIFY_H_
#define _LEAR_CLASSIFY_H_

#include <string>
#include <blitz/array.h>
#include <lear/cvision/iprocessor.h>

namespace lear {

  /**
   * Read output model file after svm learning and fills in bias and weight
   * vector. Only linear SVM output model file are supported.
   */
  class LinearClassify {
  public:
    typedef lear::IProcessor::RealType  RealType;

  public:
    // The default person detection parameters are hard coded, i.e.
    // 8x8 cell, 2x2 no. of cells in each block, 9 orientation bin
    // etc.  This default constructor simply loads the learned
    // classify model. The model is hard coded.
    LinearClassify() ;

    // Loads the linear SVM model from file 
    LinearClassify(std::string& filename, const int verbose = 0) ;

    LinearClassify(LinearClassify const &) ;

    LinearClassify& operator=(LinearClassify const &) ;

    int length() const 
    { return length_; }

    double operator()(double const *desc) const;

    float operator()(float const *desc) const;

    ~LinearClassify() {
      delete[] linearwt_;
    }

    double                   bias()       const { return linearbias_; }
    blitz::Array<RealType,1> classifier() const;

  private:
    int length_;
    double* linearwt_;
    double linearbias_;
  };

};

#endif
