/**
 * @file
 * @brief Defines Exception class.
 */
#ifndef _LEAR_EXCEPTION_H_
#define _LEAR_EXCEPTION_H_

#include <sstream>
#include <string>
#include <exception>

namespace lear {

//   Exception class
/**
 * @brief Exception class. Used for reporting error messages very briefly.
 *
 * Currently, user has to specify a where string(for e.g. function name) and mesg string(error mesg)
 * to create Exception class.
 *
 * \todo Automatic printing of Line Number and File Name where exception is thrown in code.
 */
class Exception : public std::exception {
 protected:
  std::string const mesg;
 public:
  inline Exception(std::string const mesg_) throw()
    : mesg(mesg_) {}

  inline char const *what() const throw() {
    return mesg.c_str();
  }

  inline ~Exception() throw() {}
};

#if (defined(WIN32) || defined(WIN64)) && !defined(__PRETTY_FUNCTION__)
// Perhaps this should be __FUNCDNAME__ instead.
#define __PRETTY_FUNCTION__ __FUNCSIG__
#endif
  
#define ThrowException(msgExpr)                                         \
  { std::stringstream throwExceptionMsg;                                \
    throwExceptionMsg                                                   \
      << "EXCEPTION OCCURRED: " << std::endl                            \
      << "   FUNCTION: " << __PRETTY_FUNCTION__ << std::endl            \
      << "       FILE: " << __FILE__ << std::endl                       \
      << "       LINE: " << __LINE__ << std::endl                       \
      << "    MESSAGE: " << msgExpr << std::endl;                       \
    throw lear::Exception(throwExceptionMsg.str()); }
  
} // lear
#endif // #ifndef _LEAR_EXCEPTION_H_
