#ifndef _LEAR_IMAGE_SEQ_H
#define _LEAR_IMAGE_SEQ_H

#include <memory>
#include <string>
#include <list>
#include <lear/image/imageio.h>
#include <lear/cvision/iprocessor.h>
#include <videoio/IVideo.h>

namespace lear {

  class InputImageIterator {
  public:
    typedef lear::IProcessor::RGBImage  RGBImage;
    virtual ~InputImageIterator();
    virtual bool   next() = 0;
    virtual double fps() const = 0;
    RGBImage    const &currImage() const { return currImage_; }
    std::string const &currName()  const { return currName_; }
  protected:
    RGBImage    currImage_; // should be updated by next()
    std::string currName_;  // should be updated by next()
  };

  class ImageFileInputIterator : public InputImageIterator {
  public:
    ImageFileInputIterator(std::list<std::string> const &inlist);
    virtual bool  next();
    virtual double fps() const { return -1; }
  private:
    std::list<std::string> inlist;
    std::list<std::string>::const_iterator i;
  };

  class VideoInputIterator : public InputImageIterator {
  public:
    VideoInputIterator(std::string const &fname);
    virtual bool   next();
    virtual double fps() const;
  private:
    std::auto_ptr<VideoIO::IVideo> video;
  };

  /**
   * Creates an object which iterates over a collection of input
   * images.  The caller is responsible for deleting the returned
   * object.
   *
   * If spec is a directory name, iteration is over all individual
   * image files within that directory.
   *
   * If spec is recognized as the name of a video file, iteration is
   * over all frames in the video.
   *
   * If spec is the name of an individual image file, iteration is
   * over just that single file.
   *
   * If spec is the name of a text file, we expect one filename per
   * line in the text file and iteration is over each of the named
   * individual image files.
   */
  InputImageIterator *createInputImageIterator(std::string const &spec, 
                                               std::string const &ext="");
};

#endif
