#include <lear/image/inimageiter.h>
#include <lear/image/imageio.h>
#include <lear/util/fileutil.h>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <iostream>
#include "../lib/cfg.h"
#include <videoio/FfmpegIVideo.h>

using namespace lear;
using namespace std;

lear::InputImageIterator::~InputImageIterator() {}

lear::ImageFileInputIterator::
ImageFileInputIterator(std::list<std::string> const &inlist) :
  inlist(inlist.begin(), inlist.end()), i(this->inlist.begin())
{ }

bool lear::ImageFileInputIterator::next() {
  if (i == inlist.end()) return false;
  lear::ImageIO::read(*i, currImage_);
  currName_ = *i;
  ++i;
  return true;
}

lear::VideoInputIterator::VideoInputIterator(std::string const &fname)
{
  VideoIO::KeyValueMap kvm;
  kvm["filename"] = fname;
#ifdef HAVE_FFMPEG
  this->video.reset(new VideoIO::FfmpegIVideo());
#else
  ThrowException("No available video reader class");
#endif
  this->video->open(kvm);
}
 
bool lear::VideoInputIterator::next() {
  if (!video->next()) return false;

  int const w = video->width();
  int const h = video->height();
  currImage_.resize(w,h);
  VideoIO::IVideo::Frame const &frame = video->currFrame();
  int i=0;
  for (int c=0; c<3; c++) {
    for (int x=0; x<w; x++) {
      for (int y=0; y<h; y++) {
        currImage_(x,y)[c] = frame[i++]; //frame[y+(x+c*w)*h];
      }
    }
  }

  char fname[8192];
  sprintf(fname, "%s:%09d", video->filename().c_str(), video->currFrameNum());
  currName_ = fname;

  return true;
}

double lear::VideoInputIterator::fps() const {
  return this->video->fps();
}

lear::InputImageIterator *
lear::createInputImageIterator(std::string const &spec, 
                               std::string const &ext)
{
  namespace fs=boost::filesystem;
  // my/image/dir/
  if (fs::is_directory(fs::path(spec, fs::native))) {
    list<string> inlist;
    lear::dirlist(inlist, spec, ext);
    return new ImageFileInputIterator(inlist);
  } 

  // my/dir/myvideo.avi
  try {
    return new VideoInputIterator(spec);
  } catch (std::exception const &e) {
    // Fall through...
  }

  // my/dir/myimage.jpg
  {
    std::string format = lear::ImageIO::imageFormat(spec);
    if (format.size()) {
      list<string> inlist;
      inlist.push_back(spec);
      return new ImageFileInputIterator(inlist);
    }
  }

  // my/dir/myimages.lst
  {
    list<string> inlist;
    dirlist(inlist, spec, ext); 
    return new ImageFileInputIterator(inlist);
  }
}
