
dnl Compiler flags preset
AC_DEFUN([AC_CXX_FLAGS_PRESET],[

dnl Declare variables which we want substituted in the Makefile.in's

dnl AC_SUBST(CXXFLAGS)
AC_SUBST(CXX_OPTIMIZE_FLAGS)
AC_SUBST(CXX_DEBUG_FLAGS)
AC_SUBST(CXX_PROFIL_FLAGS)
AC_SUBST(USE_SSE_FLAG)
AC_SUBST(CXX_LIBS)
AC_SUBST(AR)
AC_SUBST(AR_FLAGS)
AC_SUBST(LDFLAGS)
AC_SUBST(RANLIB)

dnl Set default values
AR="ar"
AR_FLAGS="-cru"
LDFLAGS=

AC_MSG_CHECKING([whether using $CXX preset flags])
AC_ARG_ENABLE(cxx-flags-preset,
AS_HELP_STRING([--enable-cxx-flags-preset],
[Enable C++ compiler flags preset @<:@default yes@:>@]),[],[enableval='yes'])

check_cpu () {
  CPUINFO=/proc/cpuinfo
  if test -n "$TEST_CPUINFO" ; then
    CPUINFO=$TEST_CPUINFO
  fi
  if test -r "$CPUINFO" -a "$CPUINFO" != " " ; then
    # on Linux (and others?) we can get detailed CPU information out of /proc
    cpuinfo="cat $CPUINFO"

    # detect CPU family
    cpu_family=`$cpuinfo | grep 'family' | cut -d ':' -f 2 | cut -d ' ' -f 2 | head -1`
    if test -z "$cpu_family" ; then
      cpu_family=`$cpuinfo | grep 'cpu' | cut -d ':' -f 2 | cut -d ' ' -f 2 | head -1`
    fi

    # detect CPU vendor and model
    cpu_vendor=`$cpuinfo | grep 'vendor_id' | cut -d ':' -f 2 | cut -d ' ' -f 2 | head -1`
    model_name=`$cpuinfo | grep 'model name' | cut -d ':' -f 2 | head -1`
    if test -z "$model_name" ; then
      model_name=`$cpuinfo | grep 'cpu model' | cut -d ':' -f 2 | head -1`
    fi

    # fallback: get CPU model from uname output
    if test -z "$model_name" ; then
      model_name=`uname -m`
    fi

    # parse CPU flags
    for flag in `$cpuinfo | grep '^flags' | sed -e 's/^flags.*: //' -e 's/[^a-zA-Z0-9_ ]/_/g'`; do 
          eval cpu_flag_$flag=yes
    done
  else
    # Fallback when there is no /proc/cpuinfo
    CPUINFO=" "
    case "`uname -s`" in
      FreeBSD|OpenBSD)
        cpu_family=`uname -m`;
        model_name=`sysctl -n hw.model`
        ;;
      Darwin)
        cpu_family=`uname -p`
        model_name=`machine`
        ;;
      *)
        cpu_family=`uname -m`;
        model_name=`uname -p`;
        ;;
    esac
  fi

  # detect CPU shortname as used by gcc options 
  # this list is not complete, feel free to add further entries
  cpu_arg=""
  case "$cpu_family--$model_name" in
    # DEC Alpha
    Alpha*EV6*)
      cpu_arg="ev6";
      ;;
      
    # Intel ia32
    *Intel*Core*|*Xeon*|*XEON*|*X[eE][oO][nN]*) # redundant options included for compatibility with older sh interpreters
      # XEons are 
      if test -z "$cpu_flag_lm"; then
        # No long-mode flag: it must be pentium4
        cpu_arg="pentium4";
      elif test -z "$cpu_flag_ssse3"; then
        # No Supplemental SSE3 means older than Woodcrest / Core 2
        cpu_arg="nocona";
      else
        # GCC 4.2 (what we're using right now) doesn't seem to 
        # have full support for core2 and later processors yet.
        # For now, we'll just tell GCC to be as aggressive as it
        # can be.  When we have 4.3 or later, we can change this
        # to "core2" and put additional tests in.
        cpu_arg="native";
      fi
      ;;
    *Pentium*4*Mobile*)
      cpu_arg="pentium4m";
    ;;
    *Pentium*4*)
      cpu_arg="pentium4";
      ;;
    *Pentium*III*Mobile*)
      cpu_arg="pentium3m";
    ;;
    *Pentium*III*)
      cpu_arg="pentium3";
    ;;
    *Pentium*M*pro*)
      cpu_arg="pentium-m";
    ;;
    *Celeron\(R\)*\ M*)
      cpu_arg="pentium-m";
    ;;
    *Celeron*Coppermine*)
      cpu_arg="pentium3"
    ;;
    *Celeron\(R\)*)
      cpu_arg="pentium4"
    ;;
    *Celeron*)
      cpu_arg="pentium2";
    ;;
    *Athlon*64*)
      cpu_arg="athlon64";
      ;;
    *Athlon*)
      cpu_arg="athlon";
      ;;
    *Opteron*)
      cpu_arg="opteron";
      ;;
    # MacOSX / Intel  
    *i386*i486*)
      cpu_arg="pentium-m";
      ;;
    #Core 2 Duo  
    *Intel*Core\(TM\)2*)
      cpu_arg="nocona";
      ;;

    # Intel ia64
    *Itanium*)
      # Don't need to set any flags for itanium(at the moment)
      cpu_arg="";
      ;;

    #
    *ppc*)
      cpu_arg='powerpc'
      ;;
   
    *powerpc*)
      cpu_arg='powerpc'
      ;;

    # unknown
    *)
      cpu_arg="";
      ;;
  esac

  if test -z "$cpu_arg" ; then
    if test "$CPUINFO" != " " ; then
      # fallback to uname if necessary
      TEST_CPUINFO=" "
      check_cpu_cflags=""
      check_cpu
      return
    fi
    echo "BUILD/check-cpu: Oops, could not find out what kind of cpu this machine is using." >&2
    check_cpu_cflags=""
    return
  fi

  # different compiler versions have different option names
  # for CPU specific command line options
  if test -z "$CC" ; then
    cc="gcc";
  else
    cc=$CC
  fi

  cc_ver=`$cc --version | sed 1q`
  cc_verno=`echo $cc_ver | sed -e 's/^.*gcc/gcc/g; s/[^0-9. ]/ /g;	 s/^ *//g; s/ .*//g'`
#set -- `echo $cc_verno | tr '.' ' '`
  cc_major=`echo $cc_verno | cut -d. -f1`
  cc_minor=`echo $cc_verno | cut -d. -f2`
  cc_patch=`echo $cc_verno | cut -d. -f3`
  cc_comp=`expr $cc_major '*' 100 '+' $cc_minor`

  case "$cc_ver--$cc_verno" in
    *GCC*|gcc*4.3*)
        # different gcc backends (and versions) have different CPU flags
        case `gcc -dumpmachine` in
          i?86-*)
            if test "$cc_comp" -lt 304
            then
                            check_cpu_args='-mcpu=$cpu_arg'
            else
                            check_cpu_args='-mtune=$cpu_arg'
            fi
            ;;
          ppc-*)
                    check_cpu_args='-mcpu=$cpu_arg -mtune=$cpu_arg'
            ;;
          x86_64-*)
            if test "$cc_comp" -lt 304
            then
                            check_cpu_args='-mcpu=$cpu_arg'
            else
                            check_cpu_args='-mtune=$cpu_arg'
            fi
            ;;
          *)
            check_cpu_cflags=""
            return
            ;;
        esac
      ;;
    2.95.*)
      # GCC 2.95 doesn't expose its name in --version output
      check_cpu_args='-m$cpu_arg'
      ;;
    *)
      check_cpu_cflags=""
      return
      ;;
  esac

  # now we check whether the compiler really understands the cpu type
  touch __test.c

  while test "$cpu_arg" ; do
    printf "testing $cpu_arg ... " >&2
          
    # compile check
    check_cpu_cflags=`eval echo $check_cpu_args`
    if $cc -c $check_cpu_cflags __test.c 2>/dev/null; then
            echo ok >&2
        break;
    fi

    echo failed >&2
    check_cpu_cflags=""

    # if compile failed: check whether it supports a predecessor of this CPU
    # this list is not complete, feel free to add further entries
    case "$cpu_arg" in
      # Intel ia32
      native)     cpu_arg=i7          ;;
      i7)         cpu_arg=core2       ;;
      core2)      cpu_arg=nocona      ;;
      nocona)     cpu_arg=pentium4    ;; 
      prescott)   cpu_arg=pentium4    ;;
      pentium4m)  cpu_arg=pentium4    ;;
      pentium4)   cpu_arg=pentium3    ;;
      pentium3m)  cpu_arg=pentium3    ;;
      pentium3)   cpu_arg=pentium2    ;;
      pentium2)   cpu_arg=pentiumpro  ;;
      pentiumpro) cpu_arg=pentium     ;;
      pentium)    cpu_arg=i486        ;;
      i486)       cpu_arg=i386        ;;

      # power / powerPC
      7450)       cpu_arg=7400        ;;

      *)          cpu_arg=""          ;;
    esac
  done

  rm __test.*
  echo $cpu_arg
}
 
if test "$enableval" = yes ; then

	ac_cxx_flags_preset=yes

	case "$CXX" in
	*KCC) dnl KAI C++  http://www.kai.com/
		CXX_VENDOR="KAI"
		CXXFLAGS="--restrict"
		CXX_OPTIMIZE_FLAGS="+K3"
		CXX_DEBUG_FLAGS="+K0 -g"
		AR="$CXX"
		AR_FLAGS="-o"
		case "$target" in
		*sgi*) dnl SGI C backend compiler
			CXX_OPTIMIZE_FLAGS="$CXX_OPTIMIZE_FLAGS --backend -Ofast"
		;;
		*ibm*) dnl IBM xlC backend compiler
			CXX_OPTIMIZE_FLAGS="$CXX_OPTIMIZE_FLAGS -O5 --backend -qstrict --backend -qstrict_induction"
		;;
		*) dnl other C backend compiler
			CXX_OPTIMIZE_FLAGS="$CXX_OPTIMIZE_FLAGS -O"
		;;
		esac
	;;
	cxx) dnl Compaq C++  http://www.compaq.com/
		CXX_VENDOR="Compaq"
		CXX_V=`cxx -V`
		cxx_version=`expr "$CXX_V" : '.*\(@<:@0-9@:>@\)\..*'`
		cxx_release=`expr "$CXX_V" : '.*@<:@0-9@:>@\.\(@<:@0-9@:>@\).*'`
		if test $cxx_version -eq "6" -a $cxx_release -lt "3" ; then
			CXXFLAGS="-std ansi -D__USE_STD_IOSTREAM -DBZ_ENABLE_XOPEN_SOURCE -ieee -model ansi -accept restrict_keyword -nousing_std"
		else
			CXXFLAGS="-std ansi -D__USE_STD_IOSTREAM -DBZ_ENABLE_XOPEN_SOURCE -D_OSF_SOURCE -ieee -model ansi -accept restrict_keyword -nousing_std"
		fi
		CXX_OPTIMIZE_FLAGS="-fast -inline speed -nocleanup"
		CXX_DEBUG_FLAGS="-g -msg_display_tag"
		CXX_PROFIL_FLAGS="-pg -g1"
		AR="ar"
		AR_FLAGS="-rv"
		RANLIB="ar ts"
	;;
	g++) dnl GNU C++  http://gcc.gnu.org/
		CXX_VENDOR="GNU" 
		GCC_V=`g++ --version`
		gcc_version=`expr "$GCC_V" : '.* \(@<:@0-9@:>@\)\..*'`
		gcc_release=`expr "$GCC_V" : '.* @<:@0-9@:>@\.\(@<:@0-9@:>@\).*'`
		if test $gcc_version -lt "3" ; then
			CXXFLAGS="-ftemplate-depth-40"
			CXX_OPTIMIZE_FLAGS="-O2 -funroll-loops -fstrict-aliasing -fno-gcse"
		else
			CXXFLAGS=""
			CXX_OPTIMIZE_FLAGS="-O3 -funroll-loops -fomit-frame-pointer -Wall -W -pipe " 
		fi
		C_OPTIMIZE_FLAGS="-O3 -funroll-loops -fomit-frame-pointer -Wall -W -pipe " dnl -
                dnl -ffast-math -fforce-addr -funroll-loops -fmove-all-movables -fkeep-inline-functions
		C_DEBUG_FLAGS="-g"
		CXX_DEBUG_FLAGS="-g"
		CXX_PROFIL_FLAGS="-pg"

                CpuArch=`check_cpu`
                if test -n $CpuArch; then
                    C_DEBUG_FLAGS="$C_DEBUG_FLAGS -march=$CpuArch "
                    CXX_DEBUG_FLAGS="$CXX_DEBUG_FLAGS -march=$CpuArch "
                    C_OPTIMIZE_FLAGS="$C_OPTIMIZE_FLAGS  -march=$CpuArch "
                    CXX_OPTIMIZE_FLAGS="$CXX_OPTIMIZE_FLAGS  -march=$CpuArch "
                    USE_SSE_FLAG="-DUSE_SSE"
                fi
	;;
	aCC) dnl HP aCC http://www.hp.com/go/c++
		CXX_VENDOR="HP"
		CXXFLAGS="-AA"
		CXX_OPTIMIZE_FLAGS="+O2"
		CXX_DEBUG_FLAGS="-g"
		CXX_PROFIL_FLAGS="+pal"
	;;
	icpc|icc) dnl Intel icc http://www.intel.com/
dnl remark #983: operands are evaluated in unspecified order
dnl remark #1418: external definition with no prior declaration
dnl remark #383: value copied to temporary, reference to temporary used
dnl remark #444: destructor for base class "A<T>" is not virtual, if u move to Intel 8.1, remove this
dnl remark #810: conversion from "double" to "float" may lose significant bits
                CC="icc"
                CXX="icpc"
		CXX_VENDOR="Intel"
		CXXFLAGS="" dnl -strict_ansi

                WARNING_FLAGS=" -Wall -wd981 -wd383  -wd810 -wd444 "
		C_OPTIMIZE_FLAGS="-O3 -Zp16 -ip -ansi_alias -xN -axN -sox $WARNING_FLAGS -vec_report0 "
		C_DEBUG_FLAGS="-g -O0 -C $WARNING_FLAGS "

		CXX_OPTIMIZE_FLAGS="-restrict -O3 -Zp16 -ip -ansi_alias -xN -axN -sox $WARNING_FLAGS -cxxlib-icc -vec_report0 "
		CXX_DEBUG_FLAGS="-g -O0 -C $WARNING_FLAGS "
		CXX_PROFIL_FLAGS="-p"

                CpuArch=`check_cpu`
                if test -n $CpuArch; then
                    C_DEBUG_FLAGS="$C_DEBUG_FLAGS -march=$CpuArch "
                    CXX_DEBUG_FLAGS="$CXX_DEBUG_FLAGS -march=$CpuArch "
                    C_OPTIMIZE_FLAGS="$C_OPTIMIZE_FLAGS  -march=$CpuArch "
                    CXX_OPTIMIZE_FLAGS="$CXX_OPTIMIZE_FLAGS  -march=$CpuArch "
                    USE_SSE_FLAG="-DUSE_SSE"
                fi
		SPECIAL_LDFALGS="-static-libcxa -i-static " 
	;;
	xlc++*) dnl IBM C++ compiler for Darwin
		CXX_VENDOR="IBM"
		CXXFLAGS="-qrtti -D__APPLE"
		CXX_OPTIMIZE_FLAGS="-O5 -qstrict -qstrict_induction -qmaxmem=8192 -qansialias"
		CXX_DEBUG_FLAGS="-g"
		CXX_PROFIL_FLAGS="-p"
	;;      
	*xlC*) dnl IBM Visual Age C++   http://www.ibm.com/
		CXX_VENDOR="IBM"
		CXXFLAGS="-qrtti=all -D__IBM"
		CXX_OPTIMIZE_FLAGS="-O5 -qstrict -qstrict_induction -qmaxmem=8192 -qansialias"
		CXX_DEBUG_FLAGS="-g"
		CXX_PROFIL_FLAGS="-p"
	;;      
	FCC) dnl Fujitsu C++
		CXX_VENDOR="Fujitsu"
		CXXFLAGS="-D__FUJITSU"
		CXX_OPTIMIZE_FLAGS="-K fast"
		CXX_DEBUG_FLAGS="-g"
	;;
	pgCC) dnl Portland Group   http://www.pgroup.com
		CXX_VENDOR="PGI"
		CXXFLAGS=""
		CXX_OPTIMIZE_FLAGS="-O4 -Mnoframe -Mnodepchk -Minline=levels:25"
		CXX_DEBUG_FLAGS="-g -O0"
	;;
	*CC*) 
		case "$target" in
		*sgi*) dnl SGI C++  http://www.sgi.com
			CXX_VENDOR="SGI"
			CXXFLAGS="-LANG:std -LANG:restrict -no_auto_include"
			CXX_OPTIMIZE_FLAGS="-O3 -IPA -OPT:Olimit=0:alias=typed:swp=ON"
			CXX_DEBUG_FLAGS="-g"
			AR="$CXX"
			AR_FLAGS="-ar -o"
		;;
		*cray*) dnl Cray C++
			CXX_VENDOR="Cray"
			CXXFLAGS="-h instantiate=used"
			CXX_OPTIMIZE_FLAGS="-O3 -hpipeline3 -hunroll -haggress -hscalar2"
			CXX_DEBUG_FLAGS="-g"
		;;
		esac
	;;
	*) 
		ac_cxx_flags_preset=no
	;;
	esac
	AC_MSG_RESULT([yes])
else
	AC_MSG_RESULT([no])
fi

if test "$ac_cxx_flags_preset" = yes ; then
	if test "$CXX_VENDOR" = Intel ; then
		AC_MSG_NOTICE([Setting compiler flags for $CXX_VENDOR $CXX (WAHOO!!)])
	else
		AC_MSG_NOTICE([Setting compiler flags for $CXX_VENDOR $CXX])
	fi
else

	AC_MSG_NOTICE([No flags preset found for $CXX])
fi

])


