/*
* =====================================================================================
*
*       Filename:  test_library.cpp
*
*    Description:  Code to test library routine.
*
*        Version:  1.0
*        Created:  Wednesday 13 June 2007 06:23:50  IST IST
*       Revision:  none
*       Compiler:  gcc
*
*         Author:  Navneet Dalal
*        Company:  Lear / INRIA
*
* =====================================================================================
*/

#include <iostream>
#include <algorithm>
#include <iterator>

#include <blitz/array.h>

#include <lear/windowsystem.h>
#include <lear/interface/windetect.h>// change this path as appropriate.
#include <lear/image/imageio.h>

using namespace lear;

int main(int argc, char** argv) {
  using namespace std;

  if (argc < 2) {
    std::cout << "Usage: test_library <input image>" << std::endl;
    std::cout << "It dumps detections on the command line" << std::endl;
    exit(1);
  }

  // initialize the person detector. All default parameters are set for person detector.
  WinDetectClassify windetect;// use default person detector.
  RHOGDenseParam desc;
  LinearClassify classifier;// initialize it to 64x128 person detector.
  windetect.init(&desc); // initialize the descriptor computation
  blitz::Array<DATA8,3> img = ImageIO::read<DATA8,3>(argv[1]);

  // now get detections
  std::list<DetectedRegion> detections;
  windetect.test(classifier, detections, (DATA8 const *)img.data(), img.columns(), img.rows());

  //print detections
  std::copy(detections.begin(), detections.end(), std::ostream_iterator<DetectedRegion>(std::cout, "\n"));

  // dump detections to screen.
  return 0;
}


