#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include <vector>
#define _USE_MATH_DEFINES
#include <math.h>

#include <blitz/array.h>
#include <boost/bind.hpp>

#include <lear/image/imageio.h>
#include <lear/cvision/imageslider.h>
#include <lear/cvision/iprocessor.h>
#include <lear/cvision/dhistogram.h>
#include <lear/cvision/phistogram.h>
#include <lear/cvision/dnormalizer.h>

#define double float // dirty trick to get the data as floats
# ifdef _WIN32
#  pragma warning (push)
#  pragma warning (disable:4305)
# endif
# include "../lib/persondetectorwt.tcc"
# ifdef _WIN32
#  pragma warning (pop)
# endif
#undef double

#ifdef _WIN32
#define isnan _isnan
#endif

#include <pdcuda/gradients.h>
#include <pdcuda/transposedarrays.h>
#include <pdcuda/testutils.h>
#include <pdcuda/rhogdense.h>

#include <pdcuda/rhogdense_gold.h>

using namespace std;
using namespace pdcuda;

//////////////////////////////////////////////////////////////////////////////
// Preprocessing / exporting results

// This is a quick adaptation of code from testcolorgradients.cpp.
void preprocessData(Magnitude  *hMag,  Orientation  *hOri,
                    TMagnitude *&tMag, TOrientation *&tOri,
                    RGBImage &i) 
{
  typedef blitz::TinyVector<float,3> Pixel;
  pdcuda::TransposedDeviceArray2D<Pixel> rgb(&i, true);

  int const w = i.extent()[0];
  int const h = i.extent()[1];

  // Create workspace for the results
  tMag = new pdcuda::TransposedDeviceArray2D<float>(hMag, w,h);
  OrientationF hOriF;
  tOri = new pdcuda::TransposedDeviceArray2D<float>(&hOriF, w,h);
  
  // Compute the results on the GPU where it's fast (this is tested
  // with testcolorgradients.
  pdcuda::computeSpatialGradients(tMag->devicePtr(), tOri->devicePtr(), 
                                  (float const *)rgb.devicePtr(), w, h);  

  // Transfer data back to the host so we can run CPU tests on
  // identical input data.
  tMag->fromDevice();

  // GPU likes floats, CPU likes ints for orientation.
  tOri->fromDevice();
  hOri->resize(i.extent());
  for (int ii=0; ii<w*h; ii++) {
    hOri->data()[ii] = (int)hOriF.data()[ii];
  }

  cutilSafeCall(cudaThreadSynchronize());
}

void visualizeBlockDescriptors(char const *fname, BlockDescriptors const *bd, 
                               int visScale, int border)
{
  // (cy,cx,bin,y,x): last dim is most contiguous
  int const DH = bd->extent()[3]; // # block descriptors tiled horizontally on img
  int const DW = bd->extent()[4]; // # block descriptors tiled vertically on img
  RGBImage img(DW*visScale, DH*visScale);
  memset(&img(0,0), 0, sizeof(img(0,0))*img.size());

  // Iterate over all blocks
  for (int dy=0; dy<DH; dy++) {
    for (int dx=0; dx<DW; dx++) {
      
      // Iterate over the visualization area for a block
      for (int vy=border; vy<visScale-border; vy++) {
        for (int vx=border; vx<visScale-border; vx++) {
          // Note: I'm being a little sloppy here about quantizing
          // bins and maybe having half-bin misalignments.
          int o = int(atan2(vy+0.5f-visScale/2.0f, 
                            vx+0.5f-visScale/2.0f)
                      * 180/M_PI + 180);
          if (SemiCirc) o %= 180;
          o = (o * OrientBin / 180);
                         
          // Just average the cells within a block for now
          RealType votes = 0;
          for (int cy=0; cy<NumCell[1]; cy++) {
            for (int cx=0; cx<NumCell[0]; cx++) {
              RealType const binVal = (*bd)(cy,cx,o, dy, dx);
              votes += binVal;
            }
          }

          img(dx*visScale+vx, dy*visScale+vy) = 255*votes / (NumCell[0] * NumCell[1]);
        }
      }
    }
  }

  lear::ImageIO::write(fname, img);
}

//////////////////////////////////////////////////////////////////////////////
// Main

int main(int, char const **)
{
  Magnitude    hMag;
  Orientation  hOri;
  TMagnitude   *tMag = NULL;
  TOrientation *tOri = NULL;

  int   const nIter    = 5;
  float const rmsMag   = 0.05f;
  int   const visScale = 11;
  float const rmsScore = 0.01f;

  // --------------------------------------------------------------------------
  // Read the input frame
  string infile("input.png");
  RGBImage img;
  lear::ImageIO::read(infile, img);

  // --------------------------------------------------------------------------
  // Compute gradient inputs
  preprocessData(&hMag, &hOri, tMag, tOri, img);

  // --------------------------------------------------------------------------
  // Test the visualizeBlockDescriptors function.
  //   (cy,cx,bin,y,x): last dim is most contiguous
  {
    BlockDescriptors thBD(NumCell[1],NumCell[0],OrientBin, OrientBin+1,OrientBin*2+1);
    memset(&thBD(0,0,0,0,0), 0, sizeof(thBD(0,0,0,0,0))*thBD.size());
    for (int by=0; by<thBD.extent()[3]; by++) {
      for (int bx=0; bx<thBD.extent()[4]; bx++) {
        for (int cy=0; cy<NumCell[1]; cy++) {
          for (int cx=0; cx<NumCell[0]; cx++) {
            int const o = (bx+by*cx) % OrientBin;
            thBD(cy,cx,o, by,bx) = 1.f;
          }
        }
      }
    }
    for (int b=0; b<9; b++) {
      thBD(0,0,b,0,0) = 1;
    }
    visualizeBlockDescriptors("tBD.png", &thBD, 32, 1);      
  }

  // --------------------------------------------------------------------------
  // Test block descriptors on a degenerate image
  {
    RGBImage img(BlockSizeX, BlockSizeY);
    for (int x=0; x<img.extent()[0]; x++) {
      for (int y=0; y<img.extent()[1]; y++) {
        img(x,y) = x+y;
      }
    }

    Magnitude   hMag;
    Orientation hOri;
    TMagnitude   *tMag = NULL;
    TOrientation *tOri = NULL;
    preprocessData(&hMag, &hOri, tMag, tOri, img);

    BlockDescriptors ghBD, chBD;
    computeRhogDenseCPU(&chBD, &hMag, &hOri);
    computeRhogDenseGPU_b(&ghBD, (float**)NULL, tMag, tOri);

    CheckSimilar(&chBD, &ghBD, 1e-3f);

    delete tMag;
    delete tOri;
  }

  // --------------------------------------------------------------------------
  // Test the block descriptor computation
  BlockDescriptors ghBD, chBD;
  float           *dBD = NULL;
  TimeIt3(nIter, computeRhogDenseCPU, &chBD, &hMag, &hOri);
  visualizeBlockDescriptors("cBD.png", &chBD, visScale, 1);
  computeRhogDenseGPU_b(&ghBD, &dBD, tMag, tOri);
  visualizeBlockDescriptors("gBD.png", &ghBD, visScale, 1);
  TimeIt4(nIter, computeRhogDenseGPU_b, 
          (BlockDescriptors*)NULL, (float**)NULL, tMag, tOri);

  CheckSimilar(&ghBD, &chBD, rmsMag);

  delete tMag;
  delete tOri;

  // --------------------------------------------------------------------------
  // Build window descriptors and run the classifier
  FeatureVector const hClassifier(PERSON_WEIGHT_VEC);
  float         const bias = 6.6657914910925990525925044494215f; // from WinDetect.cpp
  int           const bw = chBD.extent()[4]; // how many blocks high are ?BD
  int           const bh = chBD.extent()[3]; // how many blocks wide are ?BD
  float         const minScore = -1.4f;//numeric_limits<float>::quiet_NaN();// -6.25f; // -2;
  ScoreMap   chScores;
  TScoreMap *gtScores = NULL;

  TimeIt5(nIter, computeClassificationScoresCPU, 
          &chScores, &ghBD, &hClassifier, bias, false);
  imwriteScaledTransposed("cScores.png", &chScores, minScore);

  computeClassificationScoresGPU(&gtScores, dBD, bw, bh, &hClassifier(0), bias);
  imwriteScaledTransposed("gScores.png", gtScores->hostArray(), minScore);
  TimeIt6(nIter, computeClassificationScoresGPU, 
          (TScoreMap**)NULL, dBD, bw, bh, &hClassifier(0), bias);

  CheckSimilar(gtScores->hostArray(), &chScores, rmsScore);

  cutilSafeCall(cudaFree(dBD));
  delete gtScores;

  return 0;
}

//make -j16 && ((cd test ; ./testrhogdense) ; montage -rotate 90 -geometry 466x240 -border 3 -background blue -bordercolor blue test/cScores.png test/gScores.png t.png ; display t.png)
