function entries = dirFiltered(dirname, filter)
%entries = dirFiltered(dirname, filter)
%   Works like the built-in DIR command, but filters out various types of
%   entries.  
%
%   FILTER choices
%     'nodirs'    removes any subdirectories
%     'nodots'    removes any dot files (directory entries whose name starts with '.')
%     'subdirs'   removes any entries that are not directories
%
%   FILTER may also be set to a function that takes a directory entry and
%   returns a true/false value, where true entries are kept.  
%

switch filter
  case 'nodirs',     entries = structfilt(@(x) ~x.isdir,        dir(dirname));
  case 'nodots',     entries = structfilt(@(x)  x.name(1)~='.', dir(dirname));
  case 'subdirs',    entries = structfilt(@(x)  x.isdir,        dir(dirname));
  otherwise,         entries = structfilt(filter,               dir(dirname));
end
