%% Base paths
baseDir = inriaDir('train/inria');
annoDir = fullfile(baseDir, 'annotations_orig');
posDir  = fullfile(baseDir, 'pos_full');

posNames = structfilt(@(x) ~x.isdir, dir(posDir));

%% Pick the first INRIA file to analyze
posName = posNames(1).name;
annoName = [posName(1:end-3) 'txt'];
posPath = fullfile(posDir, posName);
annoPath = fullfile(annoDir, annoName);

%% Read the image
img = imread(posPath);

%% Read the annotations
F = fopen(annoPath, 'r');
annoRaw = fread(F, '*char')';
fclose(F);

%% Parse annotations
lines = split(sprintf('\n'), annoRaw);

regexExtractionFun  = @(regex) collapse(regexp(lines,regex,'tokens'));
numberExtractionFun = @(regex) cell2mat(cellfun(@(x) cellfun(@(y) str2double(y), x), regexExtractionFun(regex), 'UniformOutput',0));
reshapeFun          = @(arr,rows) reshape(arr,[rows,numel(arr)/rows]);
parseFun            = @(regex,rows) reshapeFun(numberExtractionFun(regex), rows);

centerPointRegexp = 'Center point on object [0-9]+ "PASperson" \(X, Y\) : \(([0-9]+),\s*([0-9]+)\)';
bboxRegexp        = 'Bounding box for object [0-9]+ "PASperson" \(Xmin, Ymin\) - \(Xmax, Ymax\) : \(([0-9]+),\s*([0-9]+)\) - \(([0-9]+),\s*([0-9]+)\)';

centerPoints = parseFun(centerPointRegexp,2); % each column: [x;y]
bboxes       = parseFun(bboxRegexp,4);        % each column: [xmin;ymin;xmax;ymax]

bboxSizes = abs(bboxes(1:2,:) - bboxes(3:4,:)) + 1;
bboxAspects = bboxSizes(2,:) ./ bboxSizes(1,:);

%% Draw annotations

clf; 
imshow(img);
hold on;
plot(centerPoints(1,:)+1, centerPoints(2,:)+1, 'b+');
for i=1:size(bboxes,2)
  rectangle('Position', [bboxes(1:2,i)' bboxSizes(1:2,i)'], 'EdgeColor',[0 0 1]);
end
hold off;

%%


(centerPoints - bboxes(1:2,:)) ./ (bboxes(1:2,:) - bboxes(3:4,:))