function extractInriaAnnotations
%
% annotationsFlat.imagename{joinedIdx}
% annotationsFlat.facecenters(:,joinedIdx)
% annotationsFlat.bboxes(:,joinedIdx)
%
% annotationsHier(imgIdx).imagename
% annotationsHier(imgIdx).facecenters(:,annoIdx)
% annotationsHier(imgIdx).bboxes(:,annoIdx)
%

clc;
extract(appPath('/afs/csail/group/vision/app/data/images/pedestrianDetection/INRIAPerson/train/inria'));
extract(appPath('/afs/csail/group/vision/app/data/images/pedestrianDetection/INRIAPerson/test/inria'));

%-----------------------------------------
function extract(baseDir)

[imageNames,imagePaths,annoPaths] = getImagePaths(baseDir);
NImgs = length(imageNames);
avgAnnoPerImg = 3; % guesstimate

% define data
annotationsFlat = struct('imagename',[], 'facecenters',[], 'bboxes',[]);
annotationsHier = struct('imagename',{}, 'facecenters',[], 'bboxes',[]);

% preallocate 
annotationsFlat.imagename   = cell(NImgs*avgAnnoPerImg,1);
annotationsFlat.facecenters = zeros(2,NImgs*avgAnnoPerImg);
annotationsFlat.bboxes      = zeros(4,NImgs*avgAnnoPerImg);

annotationsHier(NImgs).imagename = ''; 

% parse annotations and populate our data structures
joinedIdx = 1;
for imgIdx=1:length(imageNames)
  if (mod(imgIdx,10)==1), fprintf('.'); end
  if (mod(imgIdx,1000)==1), fprintf('\n'); end
  [facecenters,bboxes] = parseAnnotations(annoPaths{imgIdx});
  nAnno = size(facecenters,2);
  
  jidcs = joinedIdx+(0:nAnno-1);
  for jIdx=jidcs
    annotationsFlat.imagename{jIdx}   = imageNames{imgIdx};
  end
  annotationsFlat.facecenters(:,jidcs) = facecenters;
  annotationsFlat.bboxes(:,jidcs)      = bboxes;
  joinedIdx = joinedIdx + nAnno;
  
  annotationsHier(imgIdx).imagename   = imageNames{imgIdx};
  annotationsHier(imgIdx).facecenters = facecenters;
  annotationsHier(imgIdx).bboxes      = bboxes;
end
annotationsFlat.imagename   = {annotationsFlat.imagename{1:joinedIdx-1}};
annotationsFlat.facecenters = annotationsFlat.facecenters(:,1:joinedIdx-1);
annotationsFlat.bboxes      = annotationsFlat.bboxes(:,1:joinedIdx-1);

save(fullfile(baseDir,'annotations_orig.mat'), 'annotationsFlat', 'annotationsHier');

%-----------------------------------------
function [imageNames,imagePaths,annoPaths] = getImagePaths(baseDir)
ext        = 'png';
entries    = dir(fullfile(baseDir,'pos_full',['*.' ext]));
shortImageNames = {entries.name};
imageNames = cellfun(@(f) ['pos_full/' f], shortImageNames, 'UniformOutput',0);
fnames     = cellfun(@(f) f(1:end-3),      shortImageNames, 'UniformOutput',0);
imagePaths = cellfun(@(f) fullfile(baseDir,'pos_full',        [f ext]), fnames, 'UniformOutput',0);
annoPaths  = cellfun(@(f) fullfile(baseDir,'annotations_orig',[f 'txt' ]), fnames, 'UniformOutput',0);

%-----------------------------------------
function [facecenters,bboxes] = parseAnnotations(annoPath)

% Read and extract the lines
F = fopen(annoPath, 'r');
annoRaw = fread(F, '*char')';
fclose(F);
lines = cellfilt(@length, split(sprintf('\n'), annoRaw));

% Parsing functions
regexExtractionFun  = @(regex) collapse(regexp(lines,regex,'tokens'));
numberExtractionFun = @(regex) cell2mat(cellfun(@(x) cellfun(@(y) str2double(y), x), regexExtractionFun(regex), 'UniformOutput',0));
reshapeFun          = @(arr,rows) reshape(arr,[rows,numel(arr)/rows]);
parseFun            = @(regex,rows) reshapeFun(numberExtractionFun(regex), rows);

% Parsing regular expressions
centerPointRegexp = 'Center point on object [0-9]+ "PASperson" \(X, Y\) : \(([0-9]+),\s*([0-9]+)\)';
bboxRegexp        = 'Bounding box for object [0-9]+ "PASperson" \(Xmin, Ymin\) - \(Xmax, Ymax\) : \(([0-9]+),\s*([0-9]+)\) - \(([0-9]+),\s*([0-9]+)\)';

facecenters = parseFun(centerPointRegexp,2); % each column: [x;y]
bboxes      = parseFun(bboxRegexp,4);        % each column: [xmin;ymin;xmax;ymax]
