function gzoom(factor)
%gzoom(factor)
%  Gerald's hacked zoom function.  I extracted parts of Matlab's ZOOM
%  function from version R2008a to be able to zoom in on the current mouse
%  point.  That function has the ability to do it, but it is not publicly
%  exposed.  It would be nice to try to do a real (simpler) implementation
%  in the future, but this works for now...

% makes sure the hMode object exists
zoom(1);
% The extracted code wants an hMode object.  For now, we just get it
% instead of worrying about whether it's really needed.
hMode = getuimode(gcf,'Exploration.Zoom');
% Do the zoom
localZoomFactor2D(hMode,gca,factor,true);

%-------------------------------------------%
  function localZoomFactor2D(hMode,hAxesVector,zoom_factor,useCurrentPoint)
    % Most of the code here is from the original zoom function

    if zoom_factor > 0
      m = 0;
    else
      m = -1;
    end

    % Get bounding limits for zooming out.
    currLim = axis(hAxesVector);
    currXLim = get(hAxesVector,'XLim');
    currYLim = get(hAxesVector,'YLim');
    if ~iscell(currLim)
      currLim = {currLim};
      currXLim = {currXLim};
      currYLim = {currYLim};
    end
    newXLim = cell(size(currXLim));
    newYLim = cell(size(currYLim));
    newLim = cell(size(currLim));
    for i = 1:length(hAxesVector)
      hAxes = hAxesVector(i);
      viewinfo = resetplotview(hAxes,'GetStoredViewStruct');
      localStoreLimits(hAxes);
      limits = getappdata(hAxes,'zoom_zoomOrigAxesLimits');
      if ~isempty(viewinfo)
        maxbounds = [viewinfo.XLim, viewinfo.YLim];
      else % Use axes data bounds
        maxbounds = objbounds(hAxes);
        % Use current bounds, g161225, 163055
        if isempty(maxbounds)
          maxbounds = axis(hAxes);
        end
      end
      boundXLim = maxbounds(1:2);
      boundYLim = maxbounds(3:4);
      isXLog = strcmpi(get(hAxes,'XScale'),'log');
      isYLog = strcmpi(get(hAxes,'YScale'),'log');

      if isXLog
        currXLim{i} = log10(currXLim{i});
      end
      if isYLog
        currYLim{i} = log10(currYLim{i});
      end
      newXLim{i} = currXLim{i};
      newYLim{i} = currYLim{i};

      zoomConstraint = localChooseConstraint(hMode.ModeStateData.Constraint,hAxes);
      dx = diff(currXLim{i});
      dy = diff(currYLim{i});
      if useCurrentPoint
        pts = local_get_currentpoint(hAxes);
        center_x = pts(1);
        center_y = pts(2);
      else
        center_x = currXLim{i}(1) + dx/2;
        center_y = currYLim{i}(1) + dy/2;
      end

      % Calculate new x-limits
      if ~any(isinf(currXLim{i})) && (strcmpi(zoomConstraint,'horizontal') || strcmp(zoomConstraint,'none'))
        xmin = limits(1);
        xmax = limits(2);
        newdx = dx * (1/zoom_factor.^(m+1));
        newdx = min(newdx,xmax-xmin);
        % Limit zoom.
        center_x = max(center_x,xmin + newdx/2);
        center_x = min(center_x,xmax - newdx/2);
        newXLim{i} = [max(xmin,center_x-newdx/2) min(xmax,center_x+newdx/2)];

        % Check for log axes and return to linear values.
        if isXLog
          newXLim{i} = 10.^newXLim{i}(1:2);
        end
      end

      % Calculate new y-limits
      if ~any(isinf(currYLim{i})) && (strcmpi(zoomConstraint,'vertical') || strcmp(zoomConstraint,'none'))
        ymin = limits(3);
        ymax = limits(4);
        newdy = dy * (1/zoom_factor.^(m+1));
        newdy = min(newdy,ymax-ymin);
        % Limit zoom.
        center_y = max(center_y,ymin + newdy/2);
        center_y = min(center_y,ymax - newdy/2);
        newYLim{i} = [max(ymin,center_y-newdy/2) min(ymax,center_y+newdy/2)];

        % Check for log axes and return to linear values.
        if isYLog
          newYLim{i} = 10.^newYLim{i}(1:2);
        end
      end

      %Check for strangeness in the limits:
      if newXLim{i}(1) >= newXLim{i}(2)
        newXLim{i} = currXLim{i};
      end
      if newYLim{i}(1) >= newYLim{i}(2)
        newYLim{i} = currYLim{i};
      end
      newLim{i} = [newXLim{i},newYLim{i}];
    end

    localDoZoom2D(hAxesVector,currXLim,currYLim,newXLim,newYLim);
    % Register with undo/redo
    %localCreate2DUndo(hAxesVector,currLim,newLim);
  end
%-------------------------------------------%
  function p = local_get_currentpoint(ax)
    %GET_CURRENTPOINT Return equivalent linear scale current point
    p = get(ax,'currentpoint'); p = p(1,1:2);
    if strcmp(get(ax,'XScale'),'log'),
      p(1) = log10(p(1));
    end
    if strcmp(get(ax,'YScale'),'log'),
      p(2) = log10(p(2));
    end

  end
%-----------------------------------------------%
  function cons = localChooseConstraint(modeConstraint,hAx)
    % Check to see if the axes has a constraint
    localBehavior = hggetbehavior(hAx,'Zoom','-peek');
    if isempty(localBehavior)
      cons = modeConstraint;
    else
      % Reconcile the mode constraint with the axes constraint
      if ~strcmpi(localBehavior.Style,'both')
        if strcmpi(localBehavior.Style,'horizontal')
          if ~strcmpi(modeConstraint,'vertical')
            cons = 'horizontal';
          else
            cons = 'both';
          end
        else
          if ~strcmpi(modeConstraint,'horizontal')
            cons = 'vertical';
          else
            cons = 'both';
          end
        end
      else
        cons = modeConstraint;
      end
    end
  end
%---------------------------------------------------%
  function localStoreLimits(hAxes)

    limits = getappdata(hAxes,'zoom_zoomOrigAxesLimits');
    if isempty(limits)
      limits = localComputeLimits(hAxes);
      % Store the original axes limits for zoom. This is used
      % by zoom to prevent a zoom out that zooms further than
      % the original axes limits.
      setappdata(hAxes,'zoom_zoomOrigAxesLimits',limits);
    end
  end
%---------------------------------------------------%
  function limits = localComputeLimits(hAxes)

    xlim = local_get_xlim(hAxes); xmin = xlim(1); xmax = xlim(2);
    ylim = local_get_ylim(hAxes); ymin = ylim(1); ymax = ylim(2);
    limits = [xmin xmax ymin ymax];
  end
%---------------------------------------------------%
  function xlim = local_get_xlim(ax)
    %GET_XLIM Return equivalent linear scale xlim
    xlim = get(ax,'xlim');
    if strcmp(get(ax,'XScale'),'log'),
      xlim = log10(xlim);
    end
  end
%---------------------------------------------------%
  function ylim = local_get_ylim(ax)
    %GET_YLIM Return equivalent linear scale ylim
    ylim = get(ax,'ylim');
    if strcmp(get(ax,'YScale'),'log'),
      ylim = log10(ylim);
    end
  end
%-------------------------------------------%
  function localDoZoom2D(hAxesVector,currXLim,currYLim,newXLim,newYLim)
    % Check for axis equal and update axes limits as necessary

    if ~iscell(currXLim)
      currXLim = {currXLim};
      currYLim = {currYLim};
      newXLim = {newXLim};
      newYLim = {newYLim};
    end

    for i = 1:length(hAxesVector)
      hAxes = hAxesVector(i);
      if strcmp(get(hAxes,'dataaspectratiomode'),'manual')
        olddx = diff(get(hAxes,'XLim'));
        olddy = diff(get(hAxes,'YLim'));
        ratio = olddx / olddy;
        dx = newXLim{i}(2)-newXLim{i}(1);
        dy = newYLim{i}(2)-newYLim{i}(1);
        % Convert the dx and dy into pixel units
        hFig = ancestor(hAxes,'Figure');
        % Since we are always dealing with 2-D axes, convert the data units
        % into normalized units by normalizing against the current limits of
        % the data.
        ndx = dx / olddx;
        ndy = dy / olddy;
        pixPosRect = hgconvertunits(hFig,get(hAxes,'Position'),get(hAxes,'Units'),'Pixels',hFig);
        pdx = pixPosRect(1) + pixPosRect(3)*ndx;
        pdy = pixPosRect(2) + pixPosRect(4)*ndy;
        % Use the larger axis of the zoom as a base, unless the other axes
        % hasn't moved.
        if (pdy > pdx && ~isequal(newYLim,get(hAxes,'YLim'))) || ...
            isequal(newXLim,get(hAxes,'XLim'))
          diffSize = ratio * dy;
          diffDiff = dx - diffSize;
          diffHalf = diffDiff / 2;
          newXLim{i}(1) = newXLim{i}(1) + diffHalf;
          newXLim{i}(2) = newXLim{i}(2) - diffHalf;
        else
          diffSize = dx / ratio;
          diffDiff = dy - diffSize;
          diffHalf = diffDiff / 2;
          newYLim{i}(1) = newYLim{i}(1) + diffHalf;
          newYLim{i}(2) = newYLim{i}(2) - diffHalf;
        end
        %If the camera view angle is set to be manual and axis equal has
        %been set, bad things may happen:
        set(hAxes,'CameraViewAngleMode','auto');
      end

      h = findobj(hAxes,'Type','Image');
      if ~isempty(h)
        lims = objbounds(hAxes);
        x = lims(1:2);
        y = lims(3:4);
        %If we are within the bounds of the image to begin with. This is to
        %prevent odd behavior if we panned outside the bounds of the image
        if x(1) <= currXLim{i}(1) && x(2) >= currXLim{i}(2) &&...
            y(1) <= currYLim{i}(1) && y(2) >= currYLim{i}(2)
          dx = newXLim{i}(2) - newXLim{i}(1);
          if newXLim{i}(1) < x(1)
            newXLim{i}(1) = x(1);
            newXLim{i}(2) = newXLim{i}(1) + dx;
          end
          if newXLim{i}(2) > x(2)
            newXLim{i}(2) = x(2);
            newXLim{i}(1) = newXLim{i}(2) - dx;
          end
          dy = newYLim{i}(2) - newYLim{i}(1);
          if newYLim{i}(1) < y(1)
            newYLim{i}(1) = y(1);
            newYLim{i}(2) = newYLim{i}(1) + dy;
          end
          if newYLim{i}(2) > y(2)
            newYLim{i}(2) = y(2);
            newYLim{i}(1) = newYLim{i}(2) - dy;
          end
          % Edge case: If the zoom ends up outside the image bounds, clip to
          % the image.
          if newXLim{i}(1) < x(1)
            newXLim{i}(1) = x(1);
          end
          if newXLim{i}(2) > x(2)
            newXLim{i}(2) = x(2);
          end
          if newYLim{i}(1) < y(1)
            newYLim{i}(1) = y(1);
          end
          if newYLim{i}(2) > y(2)
            newYLim{i}(2) = y(2);
          end
        end
      end

      % Actual zoom operation
      newLim = [newXLim{i},newYLim{i}];
      axis(hAxes,newLim);
    end
  end

end