function [m,v] = ransacGaussian(data)

% http://en.wikipedia.org/wiki/RANSAC

minFracInConsensusSet = 0.85;
sampleSize            = 5;
niter                 = 10;
minConsensusLik       = 1e-3;

N = size(data,1);
bestErr = inf;
m=[];
v=[];
for iter = 1:niter
  maybeInliers = floor(rand(1,sampleSize)*N+1);
  [maybeM,maybeCV] = gaussFit(data, maybeInliers);
  consensusSet = false(N,1);
  consensusSet(maybeInliers) = 1;
  
  lik = gaussLik(maybeM, maybeCV, data);
  consensusSet(lik>minConsensusLik) = 1;
  
  if (sum(consensusSet) > N*minFracInConsensusSet)
    [betterM,betterCV] = gaussFit(data, consensusSet);
    betterErr = mean(gaussLik(betterM,betterCV, data(consensusSet,:)));
    if betterErr < bestErr
      m       = betterM;
      v       = betterCV;
      bestErr = betterErr;
    end
  end
end


%--------------------
function [m,v] = gaussFit(data, inliers)
m = mean(data(inliers,:));
v = cov(data(inliers,:));
