function localPath = appPath(varargin)
%localPath = appPath(linuxPath)
%  Performs virtual path translation for APP network drives.  Supply a
%  linux path that refers to data on an NFS/CIFS directory, and this
%  function attempts to translate to a Windows CIFS path on Windows.  On
%  Linux, LOCALPATH=LINUXPATH.  Relative paths pass untranslated on all
%  platforms.  
%
%  The available mappings are stored in appMappings.csv.  They should be
%  self-explanatory for now.
%  

if nargin>1
  linuxPath = fullfile(varargin{:});
else
  linuxPath = varargin{1};
end

% do nothing on non-Windows
if (~ispc), localPath = linuxPath; return; end
% do nothing on relative paths
if isempty(ismember(linuxPath(1),'/\')), localPath = linuxPath; return; end

linuxPath = strrep(linuxPath, '\', '/');

mappings = getMappings;
for i=1:size(mappings,1)
  if doSubs(mappings{i,:})
    return;
  end
end
error('Unknown path'); 

%-------------------------------
  function done = doSubs(prefix, dosPrefix)
    nL = numel(linuxPath);
    nP = numel(prefix);
    if nL < nP 
      % prefix path is more specific or on another branch
      done = 0;
    elseif ~strcmp(linuxPath(1:nP), prefix)
      % different branches
      done = 0;
    elseif (nL > nP) && (linuxPath(nP+1)~='/')
      % whole directory name didn't match
      done = 0;
    else
      done = 1;
      if nL > nP
        localPath = [strrep(dosPrefix,'/','\') '\' strrep(linuxPath(nP+2:end),'/','\')];
      else
        localPath = [strrep(dosPrefix,'/','\')];
      end
    end
  end

%-------------------------------
  function mappings = getMappings
    persistent mappings_;
    if isempty(mappings_)
      % read mappings file
      d = fileparts(which(mfilename));
      mappingsFile = fullfile(d, 'appMappings.csv');
      F=fopen(mappingsFile);
      txt = fread(F, '*char')';
      fclose(F);
      
      % break into lines
      lines = split(sprintf('\r\n'), txt);
      
      % first line gives a machine name list...find the current box
      tokens = split(',', lines{1});
      assert(strcmp(tokens{1},'linux'));
      [tf,col] = ismember(lower(getenv('COMPUTERNAME')), tokens);
      assert(tf, 'Could not find mappings for "%s" in "%s"',...
        lower(getenv('COMPUTERNAME')), mappingsFile);
      
      % extract the mappings that exist for this box
      mappings_ = cell(0,2);
      for j=2:length(lines)
        tokens = rsplit(',', lines{j});
        if length(tokens)>=col && ~isempty(tokens{col})
          mappings_{end+1,1} = tokens{1}; %#ok<AGROW>
          mappings_{end,2}   = tokens{col};
        end
      end
    end
    mappings = mappings_;
  end
end
