function genPdatass(trkfile,calibfile,outbase,varargin)

ww = warning;
warning off;
addpath /scratch/dalleyg/tracker/trunk/matlab/trk2datass
addpath /scratch/dalleyg/activityModeling/trunk/HDPTraj
addpath /scratch/dalleyg/dalleyg/activityModeling/trunk/HDPTraj

args = struct(...
    'pedestrianheight',5.75,...       % feet
    'rectscale',32,...                % rectified pixels / cell
    'maxbounds',[-50 250 -20 140],... % feet (xmin xmax ymin ymax)
    'cellsize',5,... % feet / cell
    'nori',16,...
    'stationaryspeedmph',0.01,...
    'minmedianmph',0.05,...
    'maxspeedmph',2500,... % doesn't work so well right now.  might be a bug.  large value effectively disables this filter.
    'minobs',100,...    % observations
    'minbboxdiam',8,... % feet
    'imgfile','');

args = parseArgs(varargin,args);

maxSpeed           = args.maxspeedmph * (5280/1) * (1/3600); % feet/sec
stationarySpeed    = args.stationaryspeedmph * (5280/1) * (1/3600); % feet/sec
minMedianSpeed     = args.minmedianmph *  (5280/1) * (1/3600); % feet/sec

imgfile = ''; % use the calibrationimage specified in calibfile

%%% Filenames
generator = which('genPdatass.m');

datassfile = [outbase '.mat'];
rendering  = [outbase '.jpg'];

%%% Do the work
fprintf('%s\n%s\n', trkfile, datassfile);

[datass,uids,rectified] = ...
    trk2pdatass(trkfile, calibfile, args.imgfile, ...
                args.pedestrianheight, args.rectscale, args.maxbounds,...
                args.cellsize,args.nori, ...
                stationarySpeed,minMedianSpeed,maxSpeed, ...
                args.minobs, args.minbboxdiam);

H = ceil(size(rectified,1) / args.rectscale);
W = ceil(size(rectified,2) / args.rectscale);
O = args.nori + 1;
d=cat(2,datass{:}); qq = hist(d, 1:H*W*O);
marginal = genOptTopicIm_color(qq, args.rectscale, rectified);
imwrite(marginal, rendering);

save(datassfile, 'generator',...
     'datass','uids','rectified',...
     'trkfile', 'calibfile', 'args',...
     'H','W','O');
 
warning(ww);
