function runHdp(outdir, datassfiles, varargin)

if ~exist(outdir,'dir')
  mkdir(outdir);
end

ww = warning;
warning off; %#ok<WNOFF>
addNeededPaths;

fprintf('parsing arguments...\n');
if ischar(datassfiles)
  datassfiles = {datassfiles};
end

args = struct('trackfracs',zeros(size(datassfiles))+1e-1);
for dfi=1:length(datassfiles)
  datassfile = datassfiles{dfi};
  % This is a bit of a hack, merging args from multiple files.  We should
  % really do some error checking to make sure that inconsistent args are
  % not used.
  aa = load(datassfile,'args');
  % merge the trackfrac field
  if isfield(aa.args, 'trackfrac')
    args.trackfracs(dfi) = aa.args.trackfrac;
    aa.args = rmfield(aa.args, 'trackfrac');
  end
  % merge args from all input files
  fns = fieldnames(aa.args);
  for fi=1:length(fns)
    fn = fns{fi};
    args.(fn) = aa.args.(fn);
  end
end
args=def(args,'niter',15000);
args=def(args,'maxtrackdurationsec',90);
args=def(args,'dirichletprior',0.01);
args=def(args,'fps',30);
args=def(args,'minuniqueobspertrack',32);
args = parseArgs(varargin,args);

fprintf('loading and subsampling data...\n');
mergedRectified = [];
for dfi=1:length(datassfiles)
  datassfile = datassfiles{dfi};
  trackfrac  = args.trackfracs(dfi);
  
  load(datassfile,...
       'datass','uids','rectified',...
       'trkfile', 'calibfile', ...
       'H','W','O');

  if isempty(mergedRectified)
    mergedRectified = rectified;
  else
    nonblack = repmat(~logical(sum(rectified==0,3)), [1 1 3]);
    mergedRectified(nonblack) = rectified(nonblack);
  end

  % Remove tracks that last too long (they're probably from stable false
  % postive detections).
  shortSel = cellfun(@(t)length(t)<args.fps*args.maxtrackdurationsec, datass); %#ok<USENS> 
  % Remove tracks that are to focused on a small number of observations.
  % They're probably either loiterers that we want to ignore or stable
  % false positives.
  %shortSel = shortSel & cellfun(@(obs)2^entropy(obs)>=args.mintrackperplexity, ...
  %                              datass);
  shortSel = shortSel & cellfun(@(obs)numel(unique(obs))>=args.minuniqueobspertrack,datass);
  datassShort = datass(shortSel); %#ok<USENS> 

  % Randomly sample a small chunk of data
  [sliceSel,sliceSel] = sort(rand(size(datassShort))); 
  sliceSel = sliceSel(1:ceil(numel(datassShort)*trackfrac));
  datassSlice = {datassShort{sliceSel}};

  shortSelByClip{dfi}    = shortSel;
  datassShortByClip{dfi} = datassShort;
  sliceSelByClip{dfi}    = sliceSel;
  datassSliceByClip{dfi} = datassSlice;
end

shortSel    = cat(2,shortSelByClip{:});
datassShort = cat(2,datassShortByClip{:});
sliceSel    = cat(2,sliceSelByClip{:});
datassSlice = cat(2,datassSliceByClip{:});

% Visualize the sample marginal distribution of words
fprintf('pword renderings...\n');
d=cat(2,datassSlice{:}); qqSlice = hist(d, 1:H*W*O);
d=cat(2,datassShort{:}); qq      = hist(d, 1:H*W*O);
imwrite(genOptTopicIm_color(qqSlice, args.rectscale, mergedRectified), sprintf('%s/pword_sliced.png',outdir));
imwrite(genOptTopicIm_color(qq, args.rectscale, mergedRectified), sprintf('%s/pword.png',outdir));

fprintf('saving inputs...\n');
generator = which('runHdp.m');
save([outdir '/input.mat'], 'generator', ...
     'outdir', 'datassfiles', 'args',...
     'datassShort', 'datassSlice','shortSel', 'sliceSel', ...
     'datassShortByClip', 'datassSliceByClip','shortSelByClip', 'sliceSelByClip', ...
     'qq', 'qqSlice', ...
     'mergedRectified');

fprintf('learning the cluster model...\n');
clusterTrajByHDP(datassSlice, args.rectscale, ...
                 args.dirichletprior, args.niter, mergedRectified, ...
                 [],[],[], outdir);
fprintf('done.\n');

warning(ww);

%%%%
function args=def(args,fld,val)
if ~isfield(args,fld)
  args.(fld) = val;
end

function x=nonan(x)
x(isnan(x)) = 0;

function e=entropy(obs)
h = hist(obs/sum(obs)); h = nonan(h/sum(h)); % normalized histogram
e = sum(h .* nonan(log2(h)));
