function filtered = structfilt(fun,s)
%filtered = structfilt(fun,s)
%  Filters a struct-array S.  Function FUN is evaluated on each element of
%  S.  If FUN's value is true, the element of S is retained, otherwise it
%  is discarded.  FILTERED holds the retained values.  
%
%  Note: FUN operates on each element of the struct-array S.  It does not
%  operate on each field as STRUCTFUN does.
%
%  TODO: alternatively, allow FUN to be an indicator vector or an index
%  vector.
%

keepers = false(size(s));
for i=1:numel(s)
  truthVal   = fun(s(i));
  keepers(i) = ~isempty(truthVal) && logical(truthVal);
end
filtered = s(keepers);
