function h = detPlot(results, thistitle,legtext,axislimit,fontsize, styleOffs, logy)

%function hist_roc(tpWin,fpWin, thistitle,legtext,axislimit,fontsize,logy, styleOffs)
%h = hist_roc(...
%    tpWin,fpWin, thistitle,legtext,axislimit,fontsize,logy, styleOffs)
%
%  thistitle string       title of the plot
%  legtext   cell<string> human-friendly labels for each experiment
%                           listed in parent
%  axislimit double[4]    parameter for the AXIS command
%  fontsize  double       title and legend font size, in points
%  logy      bool         should the miss rate be ploted on a log scale?


[marker,style,color] = getStyles;

N = length(results.achievableRecallSparse);

if ~exist('thistitle','var') || isempty(thistitle) 
  thistitle = '';
end
if ~exist('legtext','var') || isempty(legtext) 
  legtext = {};
end
if ~exist('axislimit','var') || isempty(axislimit) 
  axislimit = [0 1 0 1];
end
if ~exist('fontsize','var') || isempty(fontsize) 
  fontsize = 8;
end
if ~exist('styleOffs','var') || isempty(styleOffs) 
  styleOffs = 0:N-1;
end
if ~exist('logy','var') || isempty(logy)
  logy = 1;
end

if numel(styleOffs) == 1 && N > 1
  styleOffs = styleOffs + 0:N-1;
end

%%%%%%%%%%%%%%%%%%%% NOW DRAW DET curves
cla reset;

hold on;
for i=1:N
  colorindex  = mod(styleOffs(i),length(color))+1;
  styleindex  = mod(styleOffs(i),length(style))+1;
  markerindex = mod(styleOffs(i),length(marker))+1;

  fpWin = results.falsePos{i} / results.neg{i};
  tpWin = results.truePos{i} / results.pos{i};
  
  % subsample the data if necessary
  if (length(fpWin) > 50)
    [ox,I] = subsample_logx(fpWin,20);
    oy=tpWin(I);
    oy = 1-oy;
  else
    % For many tests, our rocEnvelope step removes the need for subsampling
    ox = fpWin;
    oy = tpWin;
    oy = 1-oy;
    oy(oy<0) = 1e-10;

    % Add a point that'll sit at the left edge of the log plot
    xmin = axislimit(1);
    f = find(ox>=xmin,1); % where to stick the new point
    if (f>1)
      % interp1 requires no duplicates
      [oxx,II] = unique(ox, 'last');
      oyy      = oy(II);
      myy = interp1(oxx,oyy,xmin);
      
      %ox = [ox(1:f-1) ; linspace(0,ox(f),100)'       ; ox(f:end)];
      %oy = [oy(1:f-1) ; linspace(oy(f-1),oy(f),100)' ; oy(f:end)];
      ox = [ox(1:f-1) ; xmin ; ox(f:end)];
      oy = [oy(1:f-1) ; myy  ; oy(f:end)];
    end
  end

  line(ox,oy, 'Color',color(colorindex,:),...
       'LineStyle',style{styleindex},'LineWidth',2,...
       'Marker',marker(markerindex), 'MarkerSize',8 );
end
hold off;

axisfont=fontsize;
labelfont=fontsize;
set(gca,'Box','on');
set(gca,'FontSize',axisfont);

titletext=(['DET - ',thistitle]);
title(titletext,'FontSize',labelfont);grid on;
xlabel('false positives per window (FPPW)','FontSize',labelfont); grid on;
ylabel('miss rate','FontSize',labelfont); grid on;
                    
set(gca,'XScale','log');
if logy
  set(gca,'YScale','log');
  if (~isempty(legtext))
    hleg=legend(legtext{:},3);
  end
else
  if (~isempty(legtext))
    hleg=legend(legtext{:},1);
  end
end
if (~isempty(legtext))
  LEG = findobj(hleg,'type','text');
  set(LEG,'FontSize',labelfont);
end

axis(axislimit);
%set(gca,'XTick',[1e-6 1e-5 1e-4 1e-3 1e-2 1e-1])
%set(gca,'YTick',[1e-2 2e-2 5e-2 1e-1 2e-1 5e-1 1])

set(gca,'XMinorGrid','off');
set(gca,'YMinorGrid','off');
%%%%%%%%%%%%%%%%% Return figure handle now
h = gcf;

%plotedit on; % set plot editable
