function exptVisualizeProcessingChain

%% Make output location
outdir = fullfile(todaysResultsDir, 'processingChain');
mkdir(outdir);
clf;

%% Define hard-coded params for this example

nOrientBins   = 9;
cellSize      = 8;
numCells      = 2;
window        = [16 16 96-32 160-32];
descriptorloc = cellSize*[3 2];

WeightScale = 2;

%% Get a frame

img = imread(grandCentralDir('test/20080619-0710-westwall/pos_20090319_w64x128c8x8/orig069941_0100.png'));
img = img(window(2)+[0:window(4)-1], window(1)+[0:window(3)-1], :);
imwrite(img, fullfile(outdir, 'input.png'));
subplot(181); imshow(img);

gamma = uint8(double(img).^0.5 * 255/sqrt(255));
imwrite(gamma, fullfile(outdir, 'gamma.png'));
gray = double(rgb2gray(gamma))/255;
subplot(182); imshow(gamma);

[blockCentersX,blockCentersY] = meshgrid(...
  [0:cellSize:window(3)-cellSize-1]+cellSize*numCells/2,...
  [0:cellSize:window(4)-cellSize-1]+cellSize*numCells/2);
blockCenters = sub2ind([size(img,1),size(img,2)], blockCentersY, blockCentersX);
withCenters = gamma;
bcColor=[0 0 255];
for c=0:2
  withCenters(blockCenters+prod(window(3:4))*c) = bcColor(c+1);
  withCenters(blockCenters+1+prod(window(3:4))*c) = bcColor(c+1);
  withCenters(blockCenters+1+window(4)+prod(window(3:4))*c) = bcColor(c+1);
  withCenters(blockCenters+window(4)+prod(window(3:4))*c) = bcColor(c+1);
end
chosenBlockColor=[0 255 255];
for c=1:3
  withCenters(...
    [0:1]+descriptorloc(2)+cellSize*numCells/2, ...
    [0:1]+descriptorloc(1)+cellSize*numCells/2, c) = chosenBlockColor(c);
end
imwrite(withCenters, fullfile(outdir, 'withCenters.png'));
subplot(183); imshow(withCenters);

blockHighlight = drawDescriptorLoc(gamma);
imwrite(blockHighlight, fullfile(outdir, 'blockHighlight.png'));

subplot(184); imshow(blockHighlight);

%% Compute the gradients 
dxfilt = [-1 0 1];%-imfilter(fspecial('gaussian',16,2), [-1 1]);
dx = imfilter(gray, dxfilt, 'symmetric', 'same');
dy = imfilter(gray, dxfilt', 'symmetric', 'same');

s = max([abs(dx(:))', abs(dy(:))']);
dxCentered = (dx+s)/(2*s);
dyCentered = (dy+s)/(2*s);

dxHigh = drawDescriptorLoc(dxCentered);
dyHigh = drawDescriptorLoc(dyCentered);

imwrite(dxCentered, fullfile(outdir,'dx.png'));
imwrite(dyCentered, fullfile(outdir,'dy.png'));
imwrite(dxHigh,     fullfile(outdir,'dxHighlight.png'));
imwrite(dyHigh,     fullfile(outdir,'dyHighlight.png'));

subplot(285); imshow(dxHigh); colormap gray;
subplot(2,8,13); imshow(dyHigh); colormap gray;

%% Convert to polar coords

mag = (dx.^2 + dy.^2).^0.5;
ori = mod(atan2(dy,dx) + pi, pi) / pi;

mag = mag ./ max(mag(:));
ori = hsv2rgb(cat(3, ori, ones(size(ori)), mag));
imwrite(mag, fullfile(outdir, 'mag.png'));
imwrite(ori, fullfile(outdir, 'ori.png'));

[keydx,keydy] = meshgrid(linspace(-1,1,128),linspace(-1,1,128));
keymag = (keydx.^2 + keydy.^2).^0.5 / sqrt(2);
keyori = mod(atan2(keydy,keydx)+pi,pi)/pi;
keyori = hsv2rgb(cat(3, keyori, ones(size(keyori)), keymag));
imwrite(keyori, fullfile(outdir, 'orikey.png'));

oriHigh = drawDescriptorLoc(ori);
imwrite(oriHigh, fullfile(outdir, 'oriHighlight.png'));

subplot(286); imshow(keyori);
subplot(2,8,14); imshow(oriHigh);

%% Compute the HOG features

assert(numCells==2); % the code in this section is hardcoded with a 2x2 cell structure assumption.

gradMag    = (dx.^2 + dy.^2).^0.5;
gradOrient = atan2(dy,dx);

l = descriptorloc;
gradMag    = gradMag(l(2)+(0:cellSize*numCells-1), l(1)+(0:cellSize*numCells-1));
gradOrient = gradOrient(l(2)+(0:cellSize*numCells-1), l(1)+(0:cellSize*numCells-1));

gaussianStencil = fspecial('gaussian',cellSize*numCells, sqrt(2)*(cellSize*numCells/(2*WeightScale)));
imwrite(gaussianStencil./max(gaussianStencil(:)), fullfile(outdir, 'gaussianStencil.png'));
subplot(187); imshow(gaussianStencil,[]);

hog = zeros(numCells^2, nOrientBins);
for x=1:cellSize*numCells
  for y=1:cellSize*numCells
    cellIdx   = floor((x-1)/cellSize) + floor((y-1)/cellSize)*numCells + 1;
    orientIdx = mod(floor((gradOrient(y,x) + pi)*nOrientBins/pi),nOrientBins) + 1;
    % Here we use greedy voting, not the proportional voting that's favored
    % by Dalal.
    hog(cellIdx,orientIdx) = hog(cellIdx,orientIdx) + ...
      gradMag(y,x) * gaussianStencil(y,x); 
  end
end

%% Render the HOG

numCellX    = numCells;
numCellY    = numCells;
orientBin   = nOrientBins;
visScale    = 31;
cellBorder  = 2;
blockBorder = 6;
normFactor  = 64;

CellGridColor  = [0,0,255];
BlockGridColor = [0,255,255];
HistColor      = [255,255,255];

visCellWidth   = cellBorder + visScale;
visCellHeight  = cellBorder + visScale;
visBlockWidth  = blockBorder + visScale*numCellX + cellBorder*(numCellX-1);
visBlockHeight = blockBorder + visScale*numCellY + cellBorder*(numCellY-1);

hogRepr = zeros(...
  visBlockHeight + blockBorder,...
  visBlockWidth + blockBorder, 3, 'uint8');

% Draw cell gridlines
for cx=1:numCellX-1
  for border=0:cellBorder-1
    for y=0:size(hogRepr,1)-1
      for c=1:3
        hogRepr(y+1, cx*visCellWidth - border + blockBorder, c) = CellGridColor(c);
      end
    end
  end
end
for cy=1:numCellY-1
  for border=0:cellBorder-1
    for x=0:size(hogRepr,2)-1
      for c=1:3
        hogRepr(cy*visCellHeight - border + blockBorder, x+1, c) = CellGridColor(c);
      end
    end
  end
end
% Draw block gridlines
for bx=0:1
  for border=0:blockBorder-1
    for y=0:size(hogRepr,1)-1
      for c=1:3
        hogRepr(y+1, bx*visBlockWidth+border+1, c) = BlockGridColor(c);
      end
    end
  end
end
for by=0:1
  for border=0:blockBorder-1
    for x=0:size(hogRepr,2)-1
      for c=1:3
        hogRepr(by*visBlockHeight+border+1, x+1, c) = BlockGridColor(c);
      end
    end
  end
end
% Iterate over all cells      
cellIdx = 0;
for cy=0:numCellY-1
  for cx=0:numCellX-1
    cellIdx = cellIdx+1;
    % Iterate over the visualization area for a block
    for vy=0:visScale-1
      for vx=0:visScale-1
        o = mod(floor(atan2(vy+0.5-visScale/2.0,...
          vx+0.5-visScale/2.0) * 180/pi + 180), 180);
        o = floor(o * orientBin / 180);
        for c=1:3
          hogRepr(...
            cy*visCellHeight + blockBorder + vy + 1,...
            cx*visCellWidth + blockBorder + vx,...
            c) = min(255, HistColor(c) * hog(cellIdx,o+1) * normFactor);
        end
      end
    end
  end
end
imwrite(hogRepr, fullfile(outdir, 'hogRepr.png'));

figure(1);
subplot(188);
imshow(hogRepr);




%%
%-----------------------------
  function img = drawDescriptorLoc(img)
    if size(img,3)==1
      img = gray2rgb(img);
    end
    cellColor = [0 0 255];
    for ii=1:numCells-1
      for cc=1:3
        img(...
          descriptorloc(2)+(0:cellSize*numCells-1),...
          descriptorloc(1)+ii*cellSize+[-1 0], cc) = cellColor(cc);
        img(...
          descriptorloc(2)+ii*cellSize+[-1 0],...
          descriptorloc(1)+(0:cellSize*numCells-1), cc) = cellColor(cc);
      end
    end
    blockColor = [0 255 255];
    for bb=0:3
      for cc=1:3
        img(descriptorloc(2)+(-1-bb:cellSize*numCells+bb),...
          descriptorloc(1)-bb-1, cc) = blockColor(cc);
        img(descriptorloc(2)+(-1-bb:cellSize*numCells+bb),...
          descriptorloc(1)+cellSize*numCells+bb, cc) = blockColor(cc);
        img(descriptorloc(2)+cellSize*numCells+bb,...
          descriptorloc(1)+(-1-bb:cellSize*numCells+bb), cc) = blockColor(cc);
        img(descriptorloc(2)-bb-1,...
          descriptorloc(1)+(-1-bb:cellSize*numCells+bb), cc) = blockColor(cc);
      end
    end
  end

end
