function htmlRender(experiments, shortname, title, explanation)

experimentParams = struct;
experimentParams.train = experiments.train;
experimentParams.test = experiments.test;
experimentParamSet = structfun(...
  @(tt) structfun(@(elm) unique(elm), tt, 'UniformOutput',0), ...
  experimentParams , 'UniformOutput',0);

N = numel(experimentParams.train.annoset);

bash('cp', 'toggling.js', strrep(todaysResultsDir(shortname), '\','/'));

F = fopen(todaysResultsDir([shortname '/index.html']), 'w');
Fstyle = fopen(todaysResultsDir([shortname '/initialstyle.css']), 'w');
fprintf(Fstyle,'  th    { background-color:#0055cc;color:white;border: solid thin black; padding-left:8px; padding-right:8px }\n');
fprintf(Fstyle,'  table { border-spacing:0px; border: solid thin black }\n');
fprintf(Fstyle,'  td    { border: solid thin black; padding-left:2px; padding-right:2px }\n');

fprintf(F,'<html>\n');
fprintf(F,'<head>\n');
fprintf(F,'<link rel="stylesheet" type="text/css" href="initialstyle.css"/>\n');
fprintf(F,'<script src="toggling.js"></script>\n');
fprintf(F,'<script>\n');
fprintf(F,'  function updateVis() {\n');
for ttc = {'train','test'}
  tt = ttc{1};
  for fnc = fieldnames(experimentParamSet.(tt))'
    fn = fnc{1};
    fprintf(F, '    %s_%s = getSelectedValues(''%s_%s'');\n', tt,fn, tt,fn);
  end
end
fprintf(F,'    for (var i=0; i<styleNames.length; i++) {\n');
fprintf(F,'      var parts = styleNames[i].split(''_'');\n');
fprintf(F,'      if (\n');
i=0;
for ttc = {'train','test'}
  tt = ttc{1};
  for fnc = fieldnames(experimentParamSet.(tt))'
    fn = fnc{1};
    fprintf(F,'          contains(%s_%s, parts[1+%d]) &&\n', tt,fn, i);
    i=i+1;
  end
end
clear i;
fprintf(F,'          true) {\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''backgroundColor'', ''#FFffFF'');\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''color'',           ''#000000'');\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''visibility'',      ''visible'');\n');
fprintf(F,'        changecss(''img.''+styleNames[i], ''visibility'',      ''visible'');\n');
fprintf(F,'      } else {\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''backgroundColor'', ''#ccCCdd'');\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''color'',           ''#444455'');\n');
fprintf(F,'        changecss(''tr.''+styleNames[i],  ''visibility'',      ''collapse'');\n');
fprintf(F,'        changecss(''img.''+styleNames[i], ''visibility'',      ''hidden'');\n');
fprintf(F,'      }\n');
fprintf(F,'    }\n');
fprintf(F,'  }\n');
fprintf(F,'</script>\n');
fprintf(F,'</head>\n');

fprintf(F,'<body onload="updateVis();">\n');
fprintf(F,'<h1>%s</h1>\n', title);
fprintf(F,'%s\n',explanation);
fprintf(F,'<div>\n');
fprintf(F,'  <img src="aacByRank.png">\n');
fprintf(F,'</div>\n');

overlays = java.lang.StringBuffer('<img src="figs/bg.png" width="1000" height="500">');

% UI table headers
uiTable  = java.lang.StringBuffer('<table border="1">\n');
uiTable.append('<tr>');
uiTable.append(  '<th rowspan="2">AAC Rank</th>');
uiTable.append(  join('', map(@(traintest) sprintf('<th colspan="%d">%s</th>', ...
  length(fieldnames(experimentParamSet.(traintest))),...
  traintest), ...
  fieldnames(experimentParamSet))));
uiTable.append(  '<th rowspan="2">Legend</th>');
uiTable.append('</tr>\n');
uiTable.append('<tr>');
uiTable.append(  char(join('', map(@(traintest) ...
  map(@(f) sprintf('<th>%s</th>',f), ...
  fieldnames(experimentParamSet.(traintest))),...
  fieldnames(experimentParamSet)))));
uiTable.append('</tr>\n');

% create table filter controls
uiTable.append('  <tr>\n    <th>&nbsp;</th>\n');
for traintestCell = fieldnames(experimentParamSet)'
  traintest = traintestCell{1};
  for fieldCell = fieldnames(experimentParamSet.(traintest))'
    field = fieldCell{1};
    uiTable.append(sprintf('    <th><form><select multiple id="%s_%s" size="8" onchange="updateVis();">', traintest, field));
    uiTable.append(join('', ...
      map(@(val) sprintf('<option selected value="%s">%s</option>',val,val),...
      experimentParamSet.(traintest).(field))));
    uiTable.append('</select></form></th>\n');
  end
end
uiTable.append('    <th>&nbsp;</th>\n  </tr>\n');

styleNamesJs = java.lang.StringBuffer('<script>\nvar styleNames = new Array();\n');

for rank=1:N
  parts = fullcollapse(map(@(x)map(@(y)x.(y)(rank),fieldnames(x)), ...
    struct2cell(experimentParams)));
  stylename = ['STYLE_' join('_',parts)];
  styleNamesJs.append(sprintf('styleNames.push("%s");\n', stylename));
  fprintf(Fstyle, 'tr.%s  { background-color:#ccCCdd; color:#444455 }\n', stylename);
  fprintf(Fstyle, 'img.%s { visibility:hidden }\n', stylename);
  uiTable.append(sprintf('  <tr class="%s"><td>%d</td>', stylename, rank));
  legendFile = sprintf('<img src="figs/legend%03d.png" width="108" height="10">',rank);
  uiTable.append(sprintf('<td>%s</td>', parts{:}, legendFile));
  uiTable.append(sprintf('</tr>\n'));
  overlays.append(sprintf('<img class="%s" src="figs/rank%03d.png" width="1000" height="500" style="position:absolute; left:10px">\n', stylename, rank));
end
uiTable.append('</table>\n');
fprintf(F, '<div>%s</div>', char(overlays));
fprintf(F, sprintf(char(uiTable)));

styleNamesJs.append('</script>\n');
fprintf(F, char(styleNamesJs));

fprintf(F,'<hr><p style="font-size:small">Last updated: %s by %s@%s</p>\n', datestr(now,31), getenv('USERNAME'), hostname);
fprintf(F,'</body>\n');
fprintf(F,'</html>\n');
fclose(F);
fclose(Fstyle);
