function [experiments] = loadDalalHistData(basedir)
%[experiments] = loadDalalHistData(basedir)
%[experiments] = loadDetOptErrorData(basedir)
%    
%  If there are N experimental configurations, each with M(i) parameter
%  settings, then the return value has at least the following field set: 
%    
%    experiments.train.imageset                1xN cell array of strings
%    experiments.train.annoset                 1xN cell array of strings
%    cell array of strings                     1xN cell array of strings
%    experiments.train.hogId                   1xN cell array of strings
%    experiments.train.windowset               1xN cell array of strings
%    experiments.train.impl                    1xN cell array of strings
%    experiments.train.pass                    1xN cell array of strings
%    
%    experiments.test.imageset                 1xN cell array of strings
%    experiments.test.annoset                  1xN cell array of strings
%    experiments.test.windowset                1xN cell array of strings
%    
%    experiments.results.legend                1xN cell of strings
%
%    experiments.results.truePos               1xN cell of M(i)x1 ints
%    experiments.results.falsePos              1xN cell of M(i)x1 ints
%    experiments.results.falseNeg              1xN cell of M(i)x1 ints
%    experiments.results.trueNeg               1xN cell of M(i)x1 ints
%
%    experiments.results.pos                   1xN cell of ints
%    experiments.results.neg                   1xN cell of ints

% find files
fprintf('indexing results...\n');
[dummy,allfiles] = system('ls ../../experiments/*/*/*/*/*_hist.txt');
allfiles        = split([10 13], allfiles);
backups         = { allfiles{strmatch(fullfile(basedir, '_'), allfiles)} };
nonBackups      = setdiff(allfiles, backups);
posHistFilter   = @(n) strrmatch('pos_hist.txt',n);
%negHistFilter   = @(n) strrmatch('neg_hist.txt',n);
posHistFiles    = cellfilt(posHistFilter, nonBackups);
%negHistFiles    = cellfilt(negHistFilter, nonBackups); % just for debugging...not used
extractParent   = @(n) n(length(basedir)+1+1:end-length('pos_hist.txt')-1);
parentDirNames  = cellfun(extractParent, posHistFiles, 'UniformOutput',0);
legtext         = parentDirNames;

% Filename parsing
experimentsFlat = map(...
  @(path) fullcollapse(map(@(p) split('_',p), split('\/',path))), ...
  legtext);
assert(all(cellfun(@length,experimentsFlat)==9));
experimentsFlat = cat(1,experimentsFlat{:}); % make a 2D array

experiments = struct;
experiments.train = struct;
experiments.train.imageset  = {experimentsFlat{:,1}};
experiments.train.annoset   = {experimentsFlat{:,2}};
experiments.train.hogId     = {experimentsFlat{:,3}};
experiments.train.windowset = {experimentsFlat{:,4}};
experiments.train.impl      = {experimentsFlat{:,5}};
experiments.train.pass      = {experimentsFlat{:,6}};
experiments.test = struct;
experiments.test.imageset  = {experimentsFlat{:,7}};
experiments.test.annoset   = {experimentsFlat{:,8}};
experiments.test.windowset = {experimentsFlat{:,9}};

% read data
maindir   = basedir;
parents   = parentDirNames;
pfile     = 'pos_hist.txt';
nfile     = 'neg_hist.txt';

N      = length(parents);
toKeep = true(1,N);
experiments.results.legend = parentDirNames;
experiments.results.tpr = cell(1,N);
experiments.results.fpr = cell(1,N);
for i=1:N
  % Create complete path to the histogram files
  try
    [tpr,fpr] = load_roc(fullfile(maindir,parents{i},pfile),...
                         fullfile(maindir,parents{i},nfile));
    if any(isnan(tpr)), error('skip me...data file is probably still being processed'); end
    if any(isnan(fpr)), error('skip me...data file is probably still being processed'); end
    experiments.results.tpr{i} = tpr;
    experiments.results.fpr{i} = fpr;
  catch %#ok<CTCH>
    toKeep(i) = false;
  end
end

% fill in total pos, total neg using hard-coded values gleaned from
% *_list.log.
experiments.results.pos = cell(1,N);
experiments.results.neg = cell(1,N);
for i=1:N
  if strcmp(experiments.test.windowset{i}, 'all')
    if strcmp(experiments.test.annoset{i}, 'orig')
      experiments.results.pos{i} = 1132;
      experiments.results.neg{i} = 995452;
    elseif strcmp(experiments.test.annoset{i}, '20080211')
      experiments.results.pos{i} = 1178;
      experiments.results.neg{i} = 995452;
    elseif strcmp(experiments.test.annoset{i}, '20090319')
      experiments.results.pos{i} = 3808;
      experiments.results.neg{i} = 27808;
    else
      error('Unknown imageset: %s', experiments.test.annoset{i});
    end
  else
    error('Unknown windowset: %s', experiments.test.windowset{i});
  end
end

% compute the confusion matrix values
experiments.results.truePos  = cell(1,N);
experiments.results.falsePos = cell(1,N);
experiments.results.trueNeg  = cell(1,N);
experiments.results.falseNeg = cell(1,N);
for i=1:N
  experiments.results.truePos{i}  = ...
    round(experiments.results.tpr{i} * experiments.results.pos{i});
  experiments.results.falsePos{i} = ...
    round(experiments.results.fpr{i} * experiments.results.neg{i});
  experiments.results.trueNeg{i}  = ...
    experiments.results.neg{i} - experiments.results.falsePos{i};
  experiments.results.falseNeg{i} = ...
    experiments.results.pos{i} - experiments.results.truePos{i};
end
  
% filter out missing results
for ttc = fieldnames(experiments)'
  tt = ttc{1};
  for fnc = fieldnames(experiments.(tt))'
    fn = fnc{1};
    experiments.(tt).(fn) = {experiments.(tt).(fn){toKeep}};
  end
end
