function [linearBias,v] = readAlt(fname)
%[linearBias,v] = readAlt(fname)
%   Reads a .alt file.  This is an output file from svm_learn.  For now,
%   only linear classifiers are supported.
%
%EXAMPLE:
%  [l1,v1]=readAlt('../../experiments/inria20080211/64x128_H96_all/modelHard_4BiSVMLight.alt');
%  [l2,v2]=readAlt('../../experiments/inriaOrig/64x128_H96/modelHard_4BiSVMLight.alt');
%  plot([v1/1e308 v2/1e308+1]); legend('G','I');


F = fopen(fname, 'rb');

verStrBin = char(fread(F,10,'uchar')');
verStr = verStrBin(1:min(length(verStrBin),find(verStrBin==0,1,'first')-1));
assert(strcmp(verStr,'V6.01'));

verNum = fread(F,1,'int32');
assert(verNum>=200);

LINEAR_KERNEL     = 0;
POLYNOMIAL_KERNEL = 1;
RBF_KERNEL        = 2;
SIGMOID_KERNEL    = 3;
USER_KERNEL       = 4;
kernelType = fread(F,1,'int32'); assert(kernelType==LINEAR_KERNEL);
polyDegree = fread(F,1,'int32');  % ignored for linear
rbfGamma   = fread(F,1,'double'); % ignored for linear
coefLin    = fread(F,1,'double'); % not needed?
coefConst  = fread(F,1,'double'); % not needed?
customLen  = fread(F,1,'int32');
assert(fseek(F,customLen,'cof')==0);

totalWords = fread(F,1,'int32');
totalDoc   = fread(F,1,'int32'); % not needed?
svNum      = fread(F,1,'int32'); % not needed?

linearBias = fread(F,1,'double');

v          = fread(F,totalWords+1,'double');
